import java.lang.reflect.*;
import java.util.*;

public class MatchMakingTestDrive {
    Hashtable datingDB = new Hashtable();
    
    public static void main(String[] args) {
        MatchMakingTestDrive test = new MatchMakingTestDrive();
        test.drive();
    }
 
    public MatchMakingTestDrive() {
        initializeDatabase();
    }

    public void drive() {

        PersonBean joe = getPersonFromDatabase("Joe Javabean"); 

        System.out.println("===============================================");
        System.out.println("Using Owner Proxy...");
        PersonBean ownerProxy = getOwnerProxy(joe);

        System.out.println(ownerProxy.getStringRep());
        System.out.println("Owner attempting to change interests...");
        ownerProxy.setInterests("bowling, Go");
        System.out.println(ownerProxy.getStringRep());

        System.out.println("Owner attempting to set rating to 10...");
        try {
            ownerProxy.setHotOrNotRating(10);
        } catch (Exception e) {
            System.out.println("Can't set rating from owner proxy");
        }

        System.out.println();
        System.out.println("===============================================");
        System.out.println();

        System.out.println("Using Non-Owner Proxy...");

        PersonBean kelly = getPersonFromDatabase("Kelly Klosure"); 
        PersonBean nonOwnerProxy = getNonOwnerProxy(kelly);

        System.out.println(nonOwnerProxy.getStringRep());

        System.out.println("Non-owner attempting to change interests...");
        try {
            nonOwnerProxy.setInterests("bowling, Go");
        } catch (Exception e) {
            System.out.println("Can't set interests from non owner proxy");
        }

        System.out.println("Non-owner attempting to set rating to 3...");
        nonOwnerProxy.setHotOrNotRating(3);
        System.out.println(nonOwnerProxy.getStringRep());
    }





    PersonBean getOwnerProxy(PersonBean person) {
        
        return (PersonBean) Proxy.newProxyInstance( 
                person.getClass().getClassLoader(),
                person.getClass().getInterfaces(),
                new OwnerInvocationHandler(person));
    }

    PersonBean getNonOwnerProxy(PersonBean person) {
        
        return (PersonBean) Proxy.newProxyInstance(
                person.getClass().getClassLoader(),
                person.getClass().getInterfaces(),
                new NonOwnerInvocationHandler(person));
    }

    PersonBean getPersonFromDatabase(String name) {
        return (PersonBean)datingDB.get(name);
    }

    void initializeDatabase() {
        PersonBean joe = new PersonBeanImpl();
        joe.setName("Joe Javabean");
        joe.setGender("Male");
        joe.setInterests("cars, computers, music");
        joe.setHotOrNotRating(7);
        datingDB.put(joe.getName(), joe);

        PersonBean kelly = new PersonBeanImpl();
        kelly.setName("Kelly Klosure");
        kelly.setGender("Female");
        kelly.setInterests("ebay, movies, music");
        kelly.setHotOrNotRating(6);
        datingDB.put(kelly.getName(), kelly);
    }
}
